@@ -32,6 +32,7 @@ int ov_verbose= 0;
 adns_rrtype ov_type= adns_r_none;
 int ov_search=0, ov_qc_query=0, ov_qc_anshost=0, ov_qc_cname=1;
 int ov_tcp=0, ov_cname=0, ov_afflags=0, ov_v6map=0, ov_format=fmt_default;
+int ov_checkc=0;
 char *ov_id= 0;
 struct perqueryflags_remember ov_pqfr = { 1,1,1, tm_none };
 
@@ -45,11 +46,11 @@ static const struct optioninfo global_options[]= {
     "a", "asynch",         &ov_asynch, 1 },
   
   { ot_desconly, "answer/error output format and destination (see below):" },
-  { ot_value,            "Answers to stdout, errors as messages to stderr (default)",
+  { ot_value,    "Answers to stdout, errors as messages to stderr (default)",
     "Fs", "fmt-simple",    &ov_format, fmt_simple },
-  { ot_value,            "Answers and errors both to stdout in parseable format",
+  { ot_value,    "Answers and errors both to stdout in parseable format",
     "Fi", "fmt-inline",    &ov_format, fmt_inline },
-  { ot_value,            "Fully-parseable output format (default for --asynch)",
+  { ot_value,    "Fully-parseable output format (default for --asynch)",
     "Fa", "fmt-asynch",    &ov_format, fmt_asynch },
   		         
   { ot_desconly, "global verbosity level:" },
@@ -59,6 +60,13 @@ static const struct optioninfo global_options[]= {
     "Vn", "no-quiet",      &ov_verbose, 0 },
   { ot_value,            "Debugging mode",
     "Vd", "debug",         &ov_verbose, adns_if_debug },
+
+  { ot_value,            "Do not do for-developer consistency checks",
+    0, "no-checkc",         &ov_checkc, 0 },
+  { ot_value,            "Do for-developer consistency checks",
+    0, "checkc",            &ov_checkc, adns_if_checkc_freq },
+  { ot_value,            "Do for-developer consistency checks very often",
+    0, "checkc-freq",       &ov_checkc, adns_if_checkc_freq },
   		         
   { ot_desconly, "other global options:" },
   { ot_funcarg,          "Configuration to use instead of /etc/resolv.conf",
@@ -127,7 +135,7 @@ static const struct optioninfo perquery_options[]= {
   { ot_desconly, "asynchronous/pipe mode options:" },
   { ot_funcarg,          "Set <id>, default is decimal sequence starting 0",
     0, "asynch-id",        0,0, &of_asynch_id, "id" },
-  { ot_funcarg,          "Cancel the query with id <id> (no error if not found)",
+  { ot_funcarg,  "Cancel the query with id <id> (no error if not found)",
     0, "cancel-id",        0,0, &of_cancel_id, "id" },
 
   { ot_end }
